function Ax=potential_algebraic(Jx,Ji,dx)
% potential_algebraic - calculates one component of magnetic potential 
% distribution in 3D
%
% Ax = potential_algebraic(Jx,Ji,dx)
%
% Ax - one component of magnetic potential A = [Ax Ay Az] 
%      the component is 3D matrix
%
% Jx - one component of current density J = [Jx Jy Jz]
%      the component is 3D matrix
% Ji - marker of boundary condition (if 1 the current is set)
%      this is 3D matrix
% dx - size of voxel [mm]
%
% Magnetic field distribution calculation method is based 
% on the solution of differential equation - the Ampere law 
% the Ampere law in differential form -> finite difference formula
% algebraic algorithm
% apply finite difference formula for all voxels 
% and repeat this many steps until the potential distribution stabilizes
%
% Assumptions: 
%  1) magnetic permeability is uniform in region of interest
%


[K L M] = size(Jx);     % size of 3D space (matrix) in voxels
mi0 = 4*pi*1e-7;        % magnetic permeability of free space [H/m]
dx = dx*1e-3;           % [mm -> m]

% set initial magnetic potential for voxels with current (coils)
% formula ?
% Ax = 1/6*dx^2*mi0*Jx;

% finite difference formula for all voxels 
% set algebraic linear equation   Fv = b

% vector of vector potential 
v = zeros(K*M*L,1);

% linear system matrix F is declared as sparse matrix to save memory space
nelem = numel(Jx)
F = sparse(nelem,nelem);

% all diagonal elements = 1
for i=1:size(F,1)
    F(i,i) = 1;
end

% F is band diagonal matrix - band width
bw = K*L;

% boundary condition vector b
b = zeros(bw+K*M*L+bw,1);

disp('LE matrix - generation\n')

% linear system matrix F and boundary condition vector b
for k=2:M-1
    for j=2:L-1
        for i=2:K-1
                n0 = bw + sub2ind(size(Jx),i,  j,  k);
                n1 = bw + sub2ind(size(Jx),i-1,j,  k);
                n2 = bw + sub2ind(size(Jx),i+1,j,  k);
                n3 = bw + sub2ind(size(Jx),i,  j-1,k);
                n4 = bw + sub2ind(size(Jx),i,  j+1,k);
                n5 = bw + sub2ind(size(Jx),i,  j,  k-1);
                n6 = bw + sub2ind(size(Jx),i,  j,  k+1);

                F(n0,n0) =-6; 
                F(n0,n1) = 1;
                F(n0,n2) = 1;
                F(n0,n3) = 1;
                F(n0,n4) = 1;
                F(n0,n5) = 1;
                F(n0,n6) = 1;
                
                b(n0) = dx^2*mi0*Jx(i,j,k);
        end
    end
end
disp('LE matrix. end.\n')


% solve linear equation without matrix inversion (for v (potential))

% generate right upper triangular matrix
[F b] = rumatrix(size(F,1),size(F,2),F,1,b);

% solve linear system (Gauss elemineation step back) 
v = rusolve(bw,F,b);

% reshape vector v -> Ax (magnetic vector potential)
Ax = reshape(v, size(Jx));

disp('LE solved\n')

