/*********************************************************************
 * potential_iterative_c.cpp
 *
 * This file computes potential values in 3 dimension using iterative 
 * algorithm.
 * 
 * Keep in mind:
 * <> Use 0-based indexing as always in C or C++
 * <> Indexing is column-based as in Matlab (not row-based as in C)
 * <> Use linear indexing.  [x*dimy+y] instead of [x][y]
 * 
 ********************************************************************/
#include <matrix.h>
#include <mex.h>   
#include <iostream>
#include <math.h>

using namespace std;

/* Definitions to keep compatibility with earlier versions of ML */
#ifndef MWSIZE_MAX
typedef int mwSize;
typedef int mwIndex;
typedef int mwSignedIndex;

// #if (defined(_LP64) || defined(_WIN64)) && !defined(MX_COMPAT_32)
// /* Currently 2^48 based on hardware limitations */
// # define MWSIZE_MAX    281474976710655UL
// # define MWINDEX_MAX   281474976710655UL
// # define MWSINDEX_MAX  281474976710655L
// # define MWSINDEX_MIN -281474976710655L
// #else
// # define MWSIZE_MAX    2147483647UL
// # define MWINDEX_MAX   2147483647UL
// # define MWSINDEX_MAX  2147483647L
// # define MWSINDEX_MIN -2147483647L
// #endif
// #define MWSIZE_MIN    0UL
// #define MWINDEX_MIN   0UL
 #endif

#define NUMBER_OF_NEIGHBOURS  6

//define function
double** Make2DDoubleArray(int arraySizeX, int arraySizeY);

int sub2ind(int row, int col, int num_row);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

//declare variables
    mxArray *maxiter_mx, 
            *deltamin_mx, 
            *potential_maps_mx, 
            *permittivity_mx,
            *boundary_mx, 
            *iidx_neighbours_mx, 
            *potential_maps_out_mx, 
            *potential_maps_previous_mx;
    
    const mwSize *dims_boundary, 
                 *dims_potential_maps, 
                 *dims_neighbours;
      
    double  *maxiter, 
            *deltamin,             
            *potential_maps, 
            *permittivity,
            *boundary,
            *iidx_neighbours,
            *potential_maps_out,
            *potential_maps_previous;
    
    double permittivity_neigbours[NUMBER_OF_NEIGHBOURS]= {0},
           permittivity_current,
           perm_element_plus_current_perm[NUMBER_OF_NEIGHBOURS],
           perm_weight_sum_current_elment,
           perm_sum_all_neighbours;        
    
    int dimy_boundary, dimx_boundary, 
        dimx_potential, dimy_potential,
        dimy_neighbours, dimx_neighbours;
    
    int number_of_points;
    int i,j,k,l,m,n,p,t,u;
    int maxiter_iterator,iidx_val,number_of_active_neihgbours;    
    double max,substract_previus_and_present_potential_in_point;
    

//associate inputs
    maxiter_mx              = mxDuplicateArray(prhs[0]);
    deltamin_mx             = mxDuplicateArray(prhs[1]);
    potential_maps_mx       = mxDuplicateArray(prhs[2]);
    permittivity_mx         = mxDuplicateArray(prhs[3]);
    boundary_mx             = mxDuplicateArray(prhs[4]);
    iidx_neighbours_mx      = mxDuplicateArray(prhs[5]);
    potential_maps_previous_mx  = mxDuplicateArray(prhs[2]);
    
//figure out dimensions
    
    //boundary    
    dims_boundary = mxGetDimensions(boundary_mx);
    //numdims = mxGetNumberOfDimensions(boundary_mx);
    dimy_boundary = (int)dims_boundary[0]; dimx_boundary = (int)dims_boundary[1];

    //potential
    dims_potential_maps = mxGetDimensions(potential_maps_mx);
    //numdims = mxGetNumberOfDimensions(potential_maps_mx);
    dimy_potential = (int)dims_potential_maps[0]; dimx_potential = (int)dims_potential_maps[1];
    
    //iidx_neighbours
    dims_neighbours = mxGetDimensions(iidx_neighbours_mx);
    //numdims = mxGetNumberOfDimensions(iidx_neighbours_mx);
    dimy_neighbours = (int)dims_neighbours[0]; dimx_neighbours = (int)dims_neighbours[1];
       
    
    // dynamic memory alloc
    double** potential_neigbours = Make2DDoubleArray(NUMBER_OF_NEIGHBOURS, dimx_potential);  
           
    double*  potential_in_point;  
    potential_in_point = (double*) malloc(dimx_potential*sizeof(double*));  
	for(p=0;p<dimx_potential;p++)
    {
		potential_in_point[p]=0;
	}
    
    //associate outputs            
    potential_maps_out_mx = plhs[0]= mxDuplicateArray(prhs[2]);    
   

//associate pointers

    potential_maps_out = mxGetPr(potential_maps_out_mx);
    maxiter             = mxGetPr(maxiter_mx);
    deltamin            = mxGetPr(deltamin_mx);
    potential_maps      = mxGetPr(potential_maps_mx);
    permittivity        = mxGetPr(permittivity_mx);
    boundary            = mxGetPr(boundary_mx);
    iidx_neighbours     = mxGetPr(iidx_neighbours_mx);
    potential_maps_previous = mxGetPr(potential_maps_previous_mx);      
    
    
//debug
//     for(j=0;j<dimy_potential;j++) //rows
//         {
//         for(i=0;i<dimx_potential;i++) //cols
//         {        
//             //potential_maps_out[i*dimy+j]=potential_maps[i*dimy+j];
//             //mexPrintf("element[%d][%d] = %f",j,i,potential_maps[i*dimy+j]);
//             mexPrintf("%f ",potential_maps[i*dimy+j]);
//         }
//         mexPrintf("\n");
//     }

    
    //convert from double to int
    maxiter_iterator=(int)maxiter[0];
    
    //number of points equals dim y boundary
    number_of_points=dimy_boundary;
    
     for(i=0;i<maxiter_iterator;i++)
     {
            // copy potential map before each iteration
            // use to compute computation accuracy
            memcpy(potential_maps_previous, potential_maps_out, dimy_potential*dimx_potential*sizeof(double));                       
            
            
        // for each point
        for(j=0;j<number_of_points;j++)
        {       
            if (boundary[j]==-1)
            {
				l=0;
                permittivity_current=permittivity[j];
                
                //prepare permittivity and potential maps
                for(k=0;k<dimx_neighbours;k++)
                {
                    //clear 
					for(p=0;p<dimx_potential;p++)
					{					
						potential_in_point[p]=0;
					}

					iidx_val=(int)iidx_neighbours[k*dimy_neighbours+j];
                    
                    //in C we are indexing from zero
                    ///in this variable input contains MATLAB indexing
                    iidx_val=iidx_val-1;
                                                            
//                     if(iidx_val>=0)
//                     {
                        permittivity_neigbours[l]=permittivity[iidx_val];
                        for(m=0;m<dimx_potential;m++)
                        {
                            //save value for each elctrode in column
                            potential_neigbours[l][m]=potential_maps_out[m*dimy_potential + iidx_val];
                            //debug
//                             if ((j==0)&&(i==0)){
//                                 mexPrintf("pm=%f ",potential_maps[m*dimy_potential + iidx_val]);
//                                 mexPrintf("m=%d j=%d",m,j);
//                             }
                        }
                        l=l+1;

//                     }                                        
                }
           
                              
                
                //compute potential
                number_of_active_neihgbours=l;
               //add to each neighbour permittivity curent point perm value
                perm_sum_all_neighbours=0;
                for(n=0;n<number_of_active_neihgbours;n++)
                {
                    perm_element_plus_current_perm[n]=permittivity_neigbours[n]+permittivity_current;                    
                    perm_sum_all_neighbours=perm_sum_all_neighbours+permittivity_neigbours[n];
                }
                    perm_weight_sum_current_elment=perm_sum_all_neighbours+permittivity_current*number_of_active_neihgbours;                    



                //in each column are values for current electrode				
				//matrix multiplication
				//step 1 multiply elements
                for(n=0;n<number_of_active_neihgbours;n++)
                {
                    for(p=0;p<dimx_potential;p++)
                    {
                        potential_neigbours[n][p] = potential_neigbours[n][p]*perm_element_plus_current_perm[n];
                    }
                }

				//sum in each column
				//step 2 sum
                for(p=0;p<dimx_potential;p++)
                {
					for(n=0;n<number_of_active_neihgbours;n++)
					{
                        potential_in_point[p] = potential_in_point[p]+potential_neigbours[n][p];                    
					}
                }               
                    
                for(p=0;p<dimx_potential;p++)
                {
                    potential_in_point[p] = potential_in_point[p]/perm_weight_sum_current_elment;
                }

                //update potential matrix                                            
                for(m=0;m<dimx_potential;m++)
                {                    
                    potential_maps_out[m*dimy_potential + j]=potential_in_point[m];
                }
                    
            }
            
        }
             max=-1;  
            //check if given computation accuracy is reached
             for(t=0;t<dimx_potential;t++)
             {
                 for(u=0;u<dimy_potential;u++)
                 {                    
                     substract_previus_and_present_potential_in_point=abs(potential_maps_out[t*dimy_potential+u]-potential_maps_previous[t*dimy_potential+u]);
                     
                     //find max
//                      if(t==0 && u==0)
//                      {
//                          max=substract_previus_and_present_potential_in_point;
//                      }
                      
                     if (substract_previus_and_present_potential_in_point > max)
                     {
                        max = substract_previus_and_present_potential_in_point;
                     }
                 }
             }
            
            //display itearation vals
            if(i%100==0)
            {
                mexPrintf("it=%d\n",i);
                mexPrintf("delta=%+.0e\n",max);
                //mexPrintf("delta=%1.5f\n",max);
                mexEvalString("drawnow");
            }
            // calculation accuracy 
            if (max<deltamin[0])
            {
                //free memory
                for (i = 0; i < NUMBER_OF_NEIGHBOURS; i++){  
                   free(potential_neigbours[i]);  
                }  
                
                free(potential_neigbours); 

                free(potential_in_point);
                
                mexPrintf("stopping criterion: calculation precision reached\n");
                mexEvalString("drawnow");
                return;
            }
            
     }
    //stop criterion number of iteration reached
    mexPrintf("stopping criterion: maximum iteration number reached\n");
    mexEvalString("drawnow");
   
    

    
//debug
//     for(i=0;i<dimx_potential;i++)
//     {
//         for(j=0;j<dimy_potential;j++)
//         {
//             if(i==2)//if cols
//             {
//                 mexPrintf("element[%d][%d] = %2.1f\n",j,i,potential_maps_out[i*dimy+j]);
//             }
//         }
//     }
        
 //free memory
 for (i = 0; i < NUMBER_OF_NEIGHBOURS; i++){  
   free(potential_neigbours[i]);  
}  
free(potential_neigbours); 
 
free(potential_in_point);

    return;
}


// 2 domension array memory allocation
double** Make2DDoubleArray(int arraySizeX, int arraySizeY) {  
double** theArray;  
theArray = (double**) malloc(arraySizeX*sizeof(double*));  
for (int i = 0; i < arraySizeX; i++)  
   theArray[i] = (double*) malloc(arraySizeY*sizeof(double));  
   return theArray;  
}   